@@ -20,26 +20,27 @@
 #    along with ReSiStance.  If not, see <http://www.gnu.org/licenses/>.
 #########################################################################
 
+from feedmanager import FeedManager
 from pango import ELLIPSIZE_END
+from portrait import FremantleRotation
+from settings import Settings
+from threading import Thread
+from webkit import WebView
+import constants
 import feedparser
-import locale
 import gettext
 import gobject
+import gtk
 import hildon
+import os
+import osso
 import pygtk
 pygtk.require('2.0')
-import gtk
-from webkit import WebView
-import os
-import urllib2
-from portrait import FremantleRotation
-import constants
-from feedmanager import FeedManager
 # For unescape()
 import re, htmlentitydefs
-from threading import Thread
 import time
-from settings import Settings
+import urllib2
+import urllib
 
 _ = gettext.gettext
 
@@ -50,8 +51,8 @@ default_color = color_style.lookup_color(constants.DEFAULT_TEXT_COLOR)
 secondary_color = color_style.lookup_color(constants.SECONDARY_TEXT_COLOR)
 del color_style
 
-manager = FeedManager()
 settings = Settings()
+manager = FeedManager(settings)
 
 # From http://effbot.org/zone/re-sub.htm#unescape-html
 def unescape(text):
@@ -76,15 +77,16 @@ def unescape(text):
     return re.sub("&#?\w+;", fixup, text)
 
 def get_author_from_item(item):
-    words = item.title.rsplit(":")
     if 'publisher_detail' in item:
         author = item.publisher_detail.name
     elif 'author' in item:
         author = item.author
-    elif len(words) > 1:
-        author = words[0].lstrip()
     else:
-        author = ''
+        words = item.title.rsplit(":")
+        if len(words) > 1:
+            author = words[0].lstrip()
+        else:
+            author = ''
 
     return author
 
@@ -106,7 +108,7 @@ class FeedsView(hildon.GtkTreeView):
         pix_renderer = gtk.CellRendererPixbuf()
 
         # Add columns
-        column = gtk.TreeViewColumn('Icon', pix_renderer, pixbuf = 0)
+        column = gtk.TreeViewColumn('Icon', pix_renderer, pixbuf = self.FEEDS_ICON_COLUMN)
         self.append_column(column);
 
         column = gtk.TreeViewColumn('Name', text_renderer, markup = self.FEEDS_NAME_COLUMN)
@@ -117,22 +119,6 @@ class FeedsView(hildon.GtkTreeView):
         column = gtk.TreeViewColumn('Date', date_renderer, markup = self.FEEDS_READ_COLUMN)
         self.append_column(column);
 
-    def add_dummy_feed(self, url):
-
-        store = self.get_model()
-        feed_iter = store.append()
-        dummy_feed_data = feedparser.FeedParserDict()
-        dummy_feed_data.status = self.DUMMY_FEED_STATUS
-        dummy_feed_data.href = url
-        dummy_feed_data.feed = feedparser.FeedParserDict()
-        dummy_feed_data.feed.title = _('Retrieving info...')
-        store.set(feed_iter, self.FEEDS_ICON_COLUMN, None,
-                  self.FEEDS_NAME_COLUMN, self.get_feed_title_markup(dummy_feed_data),
-                  self.FEEDS_DATA_COLUMN, dummy_feed_data,
-                  self.FEEDS_READ_COLUMN, '')
-
-        return (dummy_feed_data,feed_iter)
-
     def get_feed_title_markup(self, feed_data):
         if 'subtitle' in feed_data.feed:
             secondary = feed_data.feed.subtitle
@@ -146,14 +132,6 @@ class FeedsView(hildon.GtkTreeView):
             'size="x-small">%s</span>' % \
             (feed_data.feed.title, secondary_color.to_string(), secondary)
 
-    def sort(self, order):
-        if (order == constants.ASCENDING_ORDER):
-            self.get_model().set_sort_column_id(self.FEEDS_NAME_COLUMN,
-                                                gtk.SORT_ASCENDING)
-        else:
-            self.get_model().set_sort_column_id(self.FEEDS_NAME_COLUMN,
-                                                gtk.SORT_DESCENDING)
-
 class SettingsDialog(gtk.Dialog):
 
     def __init__(self, parent):
@@ -191,8 +169,27 @@ class SettingsDialog(gtk.Dialog):
         self.vbox.add(self.auto_load_images)
         self.auto_load_images.show()
 
+        # Automatic download check button
+        self.automatic_download = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.automatic_download.set_label(_('Automatically download Podcasts'))
+        self.automatic_download.connect('clicked', self._auto_download_cb)
+        self.vbox.add(self.automatic_download)
+        self.automatic_download.show()
+
+        self.picker_download_folder = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        self.picker_download_folder.set_title(_('Auto Download Folder'))
+        self.picker_download_folder.set_value(settings.auto_download_folder)
+        self.picker_download_folder.set_alignment(0, 0.5, 0, 0)
+        self.picker_download_folder.connect('clicked', self._download_folder_cb)
+        self.picker_download_folder.set_style(hildon.BUTTON_STYLE_PICKER)
+        self.picker_download_folder.set_sensitive(settings.auto_download)
+        self.vbox.add(self.picker_download_folder)
+        self.picker_download_folder.show()
+
         # Load settings
         self.auto_load_images.set_active(settings.auto_load_images)
+        self.automatic_download.set_active(settings.auto_download)
         self.selector_orientation.set_active(0, settings.rotation_mode)
         try:
             font_size_index = constants.font_size_range.index(settings.default_font_size)
@@ -201,6 +198,27 @@ class SettingsDialog(gtk.Dialog):
             font_size_index = constants.font_size_range.index(16)
         self.selector_font_size.set_active(0, font_size_index)
 
+    def _auto_download_cb(self, button):
+        self.picker_download_folder.set_sensitive(button.get_active())
+
+    def _download_folder_cb(self, button):
+        stack = hildon.WindowStack.get_default()
+        window = stack.peek()
+
+        chooser = hildon.FileChooserDialog(window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
+        chooser.set_transient_for(window)
+        chooser.set_title(_('Select a folder for automatic downloads'))
+        chooser.set_current_folder('/home/user/MyDocs')
+
+        chooser.set_default_response(gtk.RESPONSE_OK)
+
+        response = chooser.run()
+        folder = chooser.get_filename() + '/'
+        chooser.destroy()
+
+        settings.auto_download_folder = folder
+        button.set_value(folder)
+
 class NewFeedDialog(gtk.Dialog):
 
     def __init__(self, parent):
@@ -254,32 +272,18 @@ class FindFeedsDialog(gtk.Dialog):
 
 class FeedsWindow(hildon.StackableWindow):
 
+    FEEDS_MODEL_PIXBUF_COLUMN = 0
+    FEEDS_MODEL_TITLE_COLUMN = 1
+    FEEDS_MODEL_DATA_COLUMN = 2
+    FEEDS_MODEL_READ_COLUMN = 3
+    FEEDS_MODEL_VISITS_COLUMN = 4
+
     def __init__(self):
         super(FeedsWindow, self).__init__()
 
-        # Init i18n stuff
-        languages = []
-        lc, encoding = locale.getdefaultlocale()
-        if lc:
-            languages = [lc]
-        languages += constants.DEFAULT_LANGUAGES
-        gettext.bindtextdomain(constants.RSS_COMPACT_NAME,
-                               constants.LOCALE_DIR)
-        gettext.textdomain(constants.RSS_COMPACT_NAME)
-        language = gettext.translation(constants.RSS_COMPACT_NAME,
-                                       constants.LOCALE_DIR,
-                                       languages = languages,
-                                       fallback = True)
-        _ = language.gettext
-
-        # Check that user dir exists
-        if os.path.exists(constants.RSS_CONF_FOLDER) == False:
-            os.mkdir(constants.RSS_CONF_FOLDER, 0700)
-            os.mkdir(os.path.join(constants.RSS_CONF_FOLDER, 'icons'), 0700)
-
         # Feeds
         self.view = FeedsView(gtk.HILDON_UI_MODE_NORMAL)
-        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str))
+        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str, int))
         self.view.connect ("row-activated", self._on_feed_activated)
         self.view.show()
 
@@ -316,9 +320,11 @@ class FeedsWindow(hildon.StackableWindow):
 
         # Apply settings
         settings.load()
-        self.view.sort(settings.feeds_order)
+        self._sort(settings.feeds_order)
         if settings.feeds_order == constants.DESCENDING_ORDER:
             self.descending_filter_button.set_active(True)
+        elif settings.feeds_order == constants.VISITS_ORDER:
+            self.visits_filter_button.set_active(True)
 
         # Load Feeds (could be none)
         try:
@@ -326,9 +332,38 @@ class FeedsWindow(hildon.StackableWindow):
         except IOError:
             pass
 
+    def _sort(self, order):
+        if (order == constants.ASCENDING_ORDER):
+            self.view.get_model().set_sort_column_id(self.FEEDS_MODEL_TITLE_COLUMN,
+                                                     gtk.SORT_ASCENDING)
+        elif (order == constants.DESCENDING_ORDER):
+            self.view.get_model().set_sort_column_id(self.FEEDS_MODEL_TITLE_COLUMN,
+                                                     gtk.SORT_DESCENDING)
+        else:
+            self.view.get_model().set_sort_column_id(self.FEEDS_MODEL_VISITS_COLUMN,
+                                                     gtk.SORT_DESCENDING)
+
+    def _add_dummy_feed(self, url):
+
+        store = self.view.get_model()
+        feed_iter = store.append()
+        dummy_feed_data = feedparser.FeedParserDict()
+        dummy_feed_data.status = self.view.DUMMY_FEED_STATUS
+        dummy_feed_data.href = url
+        dummy_feed_data.feed = feedparser.FeedParserDict()
+        dummy_feed_data.feed.title = _('Retrieving info...')
+        store.set(feed_iter,
+                  self.FEEDS_MODEL_PIXBUF_COLUMN, None,
+                  self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(dummy_feed_data),
+                  self.FEEDS_MODEL_DATA_COLUMN, dummy_feed_data,
+                  self.FEEDS_MODEL_READ_COLUMN, '',
+                  self.FEEDS_MODEL_VISITS_COLUMN, 0)
+
+        return (dummy_feed_data,feed_iter)
+
     def _on_feed_activated(self, treeview, path, column):
         feed_iter = self.view.get_model().get_iter(path)
-        feed_data = self.view.get_model().get_value(feed_iter, self.view.FEEDS_DATA_COLUMN)
+        feed_data = self.view.get_model().get_value(feed_iter, self.FEEDS_MODEL_DATA_COLUMN)
 
         # Check that we're not trying to open a dummy feed
         if feed_data.status == self.view.DUMMY_FEED_STATUS:
@@ -339,17 +374,21 @@ class FeedsWindow(hildon.StackableWindow):
         hildon.Program.get_instance().add_window(news_window)
         news_window.show()
 
+        # Update the visits count
+        feed_data.visits += 1
+        self.view.get_model().set_value(feed_iter, self.FEEDS_MODEL_VISITS_COLUMN, feed_data.visits)
+
         # When the news window is destroyed reset the unread cache
         news_window.connect('delete-event', self._on_news_window_destroyed,
                             gtk.TreeRowReference(self.view.get_model(), path))
 
     def _on_news_window_destroyed(self, news_window, event, row_reference):
         feed_iter = self.view.get_model().get_iter(row_reference.get_path())
-        feed_data = self.view.get_model().get_value(feed_iter, self.view.FEEDS_DATA_COLUMN)
+        feed_data = self.view.get_model().get_value(feed_iter, self.FEEDS_MODEL_DATA_COLUMN)
 
         # Reset unread cache (the value have most likely changed).
         unread_count = len([entry for entry in feed_data.entries if entry.read == False])
-        self.view.get_model().set(feed_iter, self.view.FEEDS_READ_COLUMN,
+        self.view.get_model().set(feed_iter, self.FEEDS_MODEL_READ_COLUMN,
                                   self._get_visual_unread_text(unread_count))
 
     def _on_configuration_changed(self, window, event):
@@ -385,6 +424,14 @@ class FeedsWindow(hildon.StackableWindow):
         self.descending_filter_button.set_label(_('Z-A'))
         self.descending_filter_button.connect('clicked', self._sort_descending_cb)
         menu.add_filter(self.descending_filter_button)
+        self.visits_filter_button = \
+            hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                  group = self.ascending_filter_button)
+        self.visits_filter_button.set_mode(False)
+        self.visits_filter_button.set_label(_('Favorites'))
+        self.visits_filter_button.connect('clicked', self._sort_visits_cb)
+        menu.add_filter(self.visits_filter_button)
+
 
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('New Feed'))
@@ -509,8 +556,8 @@ class FeedsWindow(hildon.StackableWindow):
         # TODO: Update counters
         store = self.view.get_model()
         for row in store:
-            unread_count = len([entry for entry in row[self.view.FEEDS_DATA_COLUMN].entries if entry.read == False])
-            row[self.view.FEEDS_READ_COLUMN] = self._get_visual_unread_text(unread_count)
+            unread_count = len([entry for entry in row[self.FEEDS_MODEL_DATA_COLUMN].entries if entry.read == False])
+            row[self.FEEDS_MODEL_READ_COLUMN] = self._get_visual_unread_text(unread_count)
 
     def _new_feed_response_cb(self, dialog, response):
 
@@ -522,7 +569,7 @@ class FeedsWindow(hildon.StackableWindow):
             hildon.hildon_gtk_window_set_progress_indicator(self, True)
 
             # Insert a dummy row while information is retrieved
-            dummy_feed_data, feed_iter = self.view.add_dummy_feed(url)
+            dummy_feed_data, feed_iter = self._add_dummy_feed(url)
             path = self.view.get_model().get_path(feed_iter)
             row_reference = gtk.TreeRowReference(self.view.get_model(), path)
 
@@ -549,7 +596,7 @@ class FeedsWindow(hildon.StackableWindow):
         # Check that the feed data was retrieved
         if new_feed_data == None:
             message = _('Could not retrieve feed')
-            dummy_data = store.get_value (feed_iter, self.view.FEEDS_DATA_COLUMN)
+            dummy_data = store.get_value (feed_iter, self.FEEDS_MODEL_DATA_COLUMN)
             if dummy_data:
                 message += 'from ' + dummy_data.href
             store.remove(feed_iter)
@@ -557,10 +604,12 @@ class FeedsWindow(hildon.StackableWindow):
             return
 
         # Update model
-        store.set(feed_iter, self.view.FEEDS_ICON_COLUMN, pixbuf,
-                  self.view.FEEDS_NAME_COLUMN, self.view.get_feed_title_markup(new_feed_data),
-                  self.view.FEEDS_DATA_COLUMN, new_feed_data,
-                  self.view.FEEDS_READ_COLUMN, self._get_visual_unread_text(len(new_feed_data.entries)))
+        store.set(feed_iter,
+                  self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf,
+                  self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(new_feed_data),
+                  self.FEEDS_MODEL_DATA_COLUMN, new_feed_data,
+                  self.FEEDS_MODEL_READ_COLUMN, self._get_visual_unread_text(len(new_feed_data.entries)),
+                  self.FEEDS_MODEL_VISITS_COLUMN, new_feed_data['visits'])
 
         # Take screenshot
         hildon.hildon_gtk_window_take_screenshot(self, False)
@@ -572,6 +621,8 @@ class FeedsWindow(hildon.StackableWindow):
             settings.auto_load_images = dialog.auto_load_images.get_active()
             settings.rotation_mode = dialog.selector_orientation.get_active(0)
             settings.default_font_size = int(dialog.selector_font_size.get_current_text())
+            settings.auto_download = dialog.automatic_download.get_active()
+
             dialog.destroy()
 
             # Update rotation
@@ -629,13 +680,12 @@ class FeedsWindow(hildon.StackableWindow):
                 feed_iter = store.append()
                 pixbuf = manager.get_favicon(feed_data.feed.link or feed_data.href)
                 unread_count = len([entry for entry in feed_data.entries if entry.read == False])
-                store.set(feed_iter, self.view.FEEDS_ICON_COLUMN, pixbuf,
-                          self.view.FEEDS_NAME_COLUMN, self.view.get_feed_title_markup(feed_data),
-                          self.view.FEEDS_DATA_COLUMN, feed_data,
-                          self.view.FEEDS_READ_COLUMN, self._get_visual_unread_text(unread_count))
-                # Improve responsiveness
-#                while gtk.events_pending():
-#                    gtk.main_iteration_do()
+                store.set(feed_iter,
+                          self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf,
+                          self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(feed_data),
+                          self.FEEDS_MODEL_DATA_COLUMN, feed_data,
+                          self.FEEDS_MODEL_READ_COLUMN, self._get_visual_unread_text(unread_count),
+                          self.FEEDS_MODEL_VISITS_COLUMN, feed_data['visits'])
 
         # If there is no data then show a dialog
         if len(feed_data_list) == 0:
@@ -654,6 +704,20 @@ class FeedsWindow(hildon.StackableWindow):
             # Disable canceling
             dialog.connect('delete-event', lambda widget,event : True)
 
+    def _find_window_urls_found_cb(self, find_window, urls):
+
+        for url in urls:
+            # Quickly show feedback to user
+            hildon.hildon_gtk_window_set_progress_indicator(self, True)
+
+            # Insert a dummy row while information is retrieved
+            dummy_feed_data, feed_iter = self._add_dummy_feed(url)
+            path = self.view.get_model().get_path(feed_iter)
+            row_reference = gtk.TreeRowReference(self.view.get_model(), path)
+
+            # Add feed to manager
+            manager.add_feed(url, self._feed_added_cb, row_reference)
+
     def _found_cb(self, key_words, dialog, feeds_info):
         if feeds_info == None:
             # Something went wrong
@@ -665,6 +729,8 @@ class FeedsWindow(hildon.StackableWindow):
         hildon.Program.get_instance().add_window(news_window)
         news_window.show()
 
+        news_window.connect('urls-found', self._find_window_urls_found_cb)
+
         dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, True)
         hildon.hildon_gtk_window_set_progress_indicator(dialog, False)
 
@@ -672,11 +738,15 @@ class FeedsWindow(hildon.StackableWindow):
 
     def _sort_ascending_cb(self, button):
         settings.feeds_order = constants.ASCENDING_ORDER
-        self.view.sort(constants.ASCENDING_ORDER)
+        self._sort(constants.ASCENDING_ORDER)
 
     def _sort_descending_cb(self, button):
         settings.feeds_order = constants.DESCENDING_ORDER
-        self.view.sort(constants.DESCENDING_ORDER)
+        self._sort(constants.DESCENDING_ORDER)
+
+    def _sort_visits_cb(self, button):
+        settings.feeds_order = constants.VISITS_ORDER
+        self._sort(constants.VISITS_ORDER)
 
     def _export_feed_cb(self, button):
         stack = hildon.WindowStack.get_default()
@@ -745,12 +815,12 @@ class EntriesView(hildon.GtkTreeView):
 
         # Add columns
         pix_renderer = gtk.CellRendererPixbuf()
-        column = gtk.TreeViewColumn('Icon', pix_renderer, pixbuf = 0)
+        column = gtk.TreeViewColumn('Icon', pix_renderer, pixbuf = self.ENTRIES_ICON_COLUMN)
         self.append_column(column);
 
         text_renderer = gtk.CellRendererText()
         text_renderer.set_property('ellipsize', ELLIPSIZE_END)
-        column = gtk.TreeViewColumn('Name', text_renderer, markup = 1)
+        column = gtk.TreeViewColumn('Name', text_renderer, markup = self.ENTRIES_NAME_COLUMN)
         column.set_expand(True)
         self.append_column(column);
 
@@ -778,9 +848,9 @@ class EntriesView(hildon.GtkTreeView):
             color = active_color.to_string()
 
         return '<span foreground="' + color + '">' \
-            + gobject.markup_escape_text(author) \
+            + gobject.markup_escape_text(unescape(author)) \
             + '</span>\n<span size="small">' \
-            + gobject.markup_escape_text(title) + '</span>'
+            + gobject.markup_escape_text(unescape(title)) + '</span>'
 
     def get_visual_entry_date(self, entry):
         if entry.has_key('updated_parsed') == False:
@@ -812,12 +882,23 @@ class EntriesView(hildon.GtkTreeView):
 
 class EntriesWindow(hildon.StackableWindow):
 
+    ENTRIES_MODEL_TEXT_COLUMN = 1
+    ENTRIES_MODEL_DATA_COLUMN = 2
+    ENTRIES_MODEL_DATE_COLUMN = 3
+
     def __init__(self, feed_data):
         super(EntriesWindow, self).__init__()
 
         self.feed_data = feed_data
         self.set_title(unescape(self.feed_data.feed.title))
 
+        self.toolbar = gtk.Toolbar()
+
+        # Edit toolbar
+        self.edit_toolbar_button_handler = 0
+        self.edit_toolbar = hildon.EditToolbar()
+        self.edit_toolbar.connect('arrow-clicked', self._restore_normal_mode)
+
         # Feeds
         self.view = EntriesView(feed_data.feed.title, gtk.HILDON_UI_MODE_NORMAL)
         self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str))
@@ -829,9 +910,9 @@ class EntriesWindow(hildon.StackableWindow):
         for entry in self.feed_data.entries:
             entry_iter = model.append()
             model.set(entry_iter,
-                      1, self.view.get_visual_entry_text(entry),
-                      2, entry,
-                      3, self.view.get_visual_entry_date(entry))
+                      self.ENTRIES_MODEL_TEXT_COLUMN, self.view.get_visual_entry_text(entry),
+                      self.ENTRIES_MODEL_DATA_COLUMN, entry,
+                      self.ENTRIES_MODEL_DATE_COLUMN, self.view.get_visual_entry_date(entry))
 
         self.align = gtk.Alignment(0, 0, 1, 1)
         self.align.set_padding(4, 0 , 16, 16)
@@ -845,29 +926,89 @@ class EntriesWindow(hildon.StackableWindow):
         self.add(self.align)
         self.align.show()
 
+        # Update feed button
+        action_box = hildon.GtkTreeView.get_action_area_box(self.view)
+        update_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        update_feed_button.set_label(_('Update feed'))
+        update_feed_button.connect('clicked', self._update_feed_cb)
+
+        action_box.pack_start(update_feed_button, True, True)
+        update_feed_button.show()
+        hildon.GtkTreeView.set_action_area_visible(self.view, True)
+
         self._create_menu()
 
+        if 'itunes' not in self.feed_data.namespaces:
+            self.download_all_button.hide()
+
+
         self.connect('configure-event', self._on_configuration_changed)
 
     def _create_menu(self):
         menu = hildon.AppMenu()
 
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Update Feed'))
-        button.connect('clicked', self._update_feed_cb)
+        button.set_label(_('Mark Entries as Read'))
+        button.connect('clicked', self._mark_read_cb, True)
         menu.append(button)
 
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Mark all as Read'))
-        button.connect('clicked', self._mark_all_read_cb)
+        button.set_label(_('Mark Entries as Not Read'))
+        button.connect('clicked', self._mark_read_cb, False)
         menu.append(button)
 
+        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button.set_label(_('Download all'))
+        button.connect('clicked', self._download_all_items_cb)
+        menu.append(button)
+        self.download_all_button = button
+
         menu.show_all()
         self.set_app_menu(menu)
 
-    def _mark_all_read_cb(self, button):
-        for entry in self.feed_data.entries:
-            entry['read'] = True
+    def _mark_read_cb(self, button, read):
+        # tree view edit mode
+        hildon.hildon_gtk_tree_view_set_ui_mode(self.view, gtk.HILDON_UI_MODE_EDIT)
+        self.view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        self.view.get_selection().unselect_all()
+
+        if read:
+            self.edit_toolbar.set_label(_('Select Feeds to mark as Read'))
+        else:
+            self.edit_toolbar.set_label(_('Select Feeds to mark as Not Read'))
+        self.edit_toolbar.set_button_label(_('Mark'))
+        if self.edit_toolbar.handler_is_connected (self.edit_toolbar_button_handler):
+            self.edit_toolbar.disconnect(self.edit_toolbar_button_handler)
+        self.edit_toolbar_button_handler = \
+            self.edit_toolbar.connect('button-clicked', self._mark_read_button_clicked_cb, read)
+
+        self.set_edit_toolbar(self.edit_toolbar)
+        self.edit_toolbar.show()
+        self.fullscreen()
+
+    def _mark_read_button_clicked_cb(self, button, read):
+        selection = self.view.get_selection()
+        selected_rows = selection.get_selected_rows()
+        model, paths = selected_rows
+        if not paths:
+            hildon.hildon_banner_show_information(self, '', _('Please select at least one Entry'))
+            return
+
+        for path in paths:
+            model[path][2]['read'] = read
+            model[path][1] = self.view.get_visual_entry_text(model[path][2])
+
+        # restore normal mode
+        self._restore_normal_mode(button)
+
+    def _restore_normal_mode(self, button):
+
+        # tree view normal mode
+        hildon.hildon_gtk_tree_view_set_ui_mode(self.view, gtk.HILDON_UI_MODE_NORMAL)
+        self.view.get_selection().unselect_all()
+
+        self.edit_toolbar.hide()
+        self.unfullscreen()
 
     def _on_entry_activated(self, treeview, path, column):
         item_window = ItemWindow(self.feed_data)
@@ -878,11 +1019,12 @@ class EntriesWindow(hildon.StackableWindow):
         item_window.show()
 
     def _item_read_cb(self, item_window, path):
-        entry = self.view.get_model()[path][2]
+        entry = self.view.get_model()[path][self.ENTRIES_MODEL_DATA_COLUMN]
 
         # Set item as read and update the UI
         entry['read'] = True
-        self.view.get_model()[path][1] = self.view.get_visual_entry_text(entry)
+        self.view.get_model()[path][self.ENTRIES_MODEL_TEXT_COLUMN] = \
+            self.view.get_visual_entry_text(entry)
 
     def _update_feed_cb(self, button):
         url = self.feed_data.href
@@ -903,12 +1045,39 @@ class EntriesWindow(hildon.StackableWindow):
         model.clear()
         for entry in self.feed_data.entries:
             entry_iter = model.append()
-            model.set(entry_iter, 1, self.view.get_visual_entry_text(entry),
-                      2, entry,
-                      3, self.view.get_visual_entry_date(entry))
+            model.set(entry_iter,
+                      self.ENTRIES_MODEL_TEXT_COLUMN, self.view.get_visual_entry_text(entry),
+                      self.ENTRIES_MODEL_DATA_COLUMN, entry,
+                      self.ENTRIES_MODEL_DATE_COLUMN, self.view.get_visual_entry_date(entry))
 
         return
 
+    def _download_all_items_cb(self, button):
+        folder = settings.auto_download_folder
+
+        urls = []
+        paths_files = []
+        for entry in self.feed_data.entries:
+            try:
+                url = entry['enclosures'][0]['href']
+                urls.append(url)
+                paths_files.append(folder + os.path.basename(urllib.url2pathname(url)))
+            except:
+                pass
+
+        hildon.hildon_gtk_window_set_progress_indicator(self, True)
+
+        manager.download_all_items(urls, paths_files, self._all_items_downloaded_cb)
+
+    def _all_items_downloaded_cb(self, downloaded):
+        if downloaded:
+            message = _('Items downloaded')
+        else:
+            message = _('Error downloading items')
+        hildon.hildon_banner_show_information(self, '', message)
+        # Remove progress information
+        hildon.hildon_gtk_window_set_progress_indicator(self, False)
+
     def _on_configuration_changed(self, window, event):
         # Change alignment padding. We don't want padding in portrait
         # Need to save space
@@ -1083,6 +1252,12 @@ class ItemWindow(hildon.StackableWindow):
         button.connect('clicked', self._item_details_cb)
         menu.append(button)
 
+        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button.set_label(_('Download'))
+        button.connect('clicked', self._download_item_cb)
+        menu.append(button)
+        self.download_button = button
+
         menu.show_all()
         self.set_app_menu(menu)
 
@@ -1137,7 +1312,7 @@ class ItemWindow(hildon.StackableWindow):
         if self.model_iter == None:
             return
 
-        item = self.model.get_value(self.model_iter, EntriesView.ENTRIES_DATA_COLUMN)
+        item = self.model.get_value(self.model_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
         self.set_item(item)
 
     def _down_button_clicked(self, button):
@@ -1147,7 +1322,7 @@ class ItemWindow(hildon.StackableWindow):
 
         self.model_iter = self.model.iter_next(self.model_iter)
 
-        item = self.model.get_value(self.model_iter, EntriesView.ENTRIES_DATA_COLUMN)
+        item = self.model.get_value(self.model_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
         self.set_item(item)
 
     def set_item_from_model(self, model, path):
@@ -1156,14 +1331,21 @@ class ItemWindow(hildon.StackableWindow):
         self.model_iter = model.get_iter(path)
 
         item_iter = model.get_iter(path)
-        item = model.get_value(item_iter, EntriesView.ENTRIES_DATA_COLUMN)
+        item = model.get_value(item_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
 
         self.item_window_header.set_item(item)
         self.set_item(item)
 
     def set_item(self, item):
 
-        self.set_title(get_author_from_item(item))
+        author = get_author_from_item(item)
+        if author == '':
+            author = self.feed_data.feed.title
+        self.set_title(author)
+
+        # hide the button if is not a podcast
+        if 'enclosures' not in item:
+            self.download_button.hide()
 
         # Set item in header
         self.item_window_header.set_item(item)
@@ -1184,7 +1366,7 @@ class ItemWindow(hildon.StackableWindow):
             body = _('No text')
 
         # Write HTML
-        self.view.load_string(body, content_type, self.feed_data.encoding, '')
+        self.view.load_string(body.encode(self.feed_data.encoding), content_type, self.feed_data.encoding, '')
 
         # Update button sensitiviness
         toolbar_button_list = self.toolbar.get_children()
@@ -1208,10 +1390,29 @@ class ItemWindow(hildon.StackableWindow):
         # Emit item read
         self.emit('item-read', self.model.get_path(self.model_iter))
 
+    def _download_item_cb(self, button):
+        item = self.item_window_header.item
+        url_file = item.enclosures[0]['href']
+        file = os.path.basename(urllib.url2pathname(url_file))
+        folder = settings.auto_download_folder
+
+        hildon.hildon_gtk_window_set_progress_indicator(self, True)
+        manager.download_item(url_file, folder+file, self._item_downloaded_cb)
+
+
+
+    def _item_downloaded_cb(self, downloaded):
+        if downloaded:
+            message = _('Item downloaded')
+        else:
+            message = _('Error downloading item')
+        hildon.hildon_banner_show_information(self, '', message)
+        # Remove progress information
+        hildon.hildon_gtk_window_set_progress_indicator(self, False)
+
 class FindView(hildon.GtkTreeView):
 
-    FIND_DATA_COLUMN = 0
-    FIND_NAME_COLUMN = 1
+    FIND_SITE_VIEW = 0
 
     def __init__(self, ui_mode):
         super(FindView, self).__init__(ui_mode)
@@ -1219,12 +1420,18 @@ class FindView(hildon.GtkTreeView):
         # Add columns
         text_renderer = gtk.CellRendererText()
         text_renderer.set_property('ellipsize', ELLIPSIZE_END)
-        column = gtk.TreeViewColumn('Name', text_renderer, markup=1)
+        column = gtk.TreeViewColumn('Name', text_renderer, markup=self.FIND_SITE_VIEW)
         column.set_expand(True)
         self.append_column(column)
 
 
 class FindWindow(hildon.StackableWindow):
+    __gsignals__ = {
+        "urls-found": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
+        }
+
+    FIND_MODEL_SITE_COLUMN = 0
+    FIND_MODEL_URL_COLUMN = 1
 
     def __init__(self, feedsinfo, key_words, feedsWindow):
         super(FindWindow, self).__init__()
@@ -1258,8 +1465,9 @@ class FindWindow(hildon.StackableWindow):
         for feed_info in self.feeds_info:
             feed_iter = store.append()
 
-            store.set(feed_iter, self.view.FIND_NAME_COLUMN, gobject.markup_escape_text(feed_info['sitename']),
-                      self.view.FIND_DATA_COLUMN, feed_info['dataurl'])
+            store.set(feed_iter,
+                      self.FIND_MODEL_SITE_COLUMN, gobject.markup_escape_text(unescape(feed_info['sitename'])),
+                      self.FIND_MODEL_URL_COLUMN, feed_info['dataurl'])
 
         self.view.show()
 
@@ -1284,20 +1492,9 @@ class FindWindow(hildon.StackableWindow):
             hildon.hildon_banner_show_information(self, '', _('Please select at least one Feed'))
             return
 
-        for path in paths:
-            url = model[path][FindView.FIND_DATA_COLUMN]
-
-            # Quickly show feedback to user
-            hildon.hildon_gtk_window_set_progress_indicator(feedsWindow, True)
-
-            # Insert a dummy row while information is retrieved
-            dummy_feed_data, feed_iter = feedsWindow.view.add_dummy_feed(url)
-            path = feedsWindow.view.get_model().get_path(feed_iter)
-            row_reference = gtk.TreeRowReference(feedsWindow.view.get_model(), path)
-
-            # Add feed to manager
-            manager.add_feed(url, feedsWindow._feed_added_cb, row_reference)
+        urls = [model[path][self.FIND_MODEL_URL_COLUMN] for path in paths]
 
+        self.emit('urls-found', urls)
 
         # restore normal mode
         self._restore_normal_mode(button)
